/****************************************************************/
/* Socket unit - TCP/UDP                                        */
/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999                  */
/****************************************************************/
#include "socks.h"
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>

#define SOCKET_VERSION "0.14"

int GetPortByName(char *port,char *proto) {
   struct servent *PSE;
   
   PSE = getservbyname(port,proto);
   if( PSE == NULL )
      return atoi(port);
   return ntohs(PSE->s_port);
}

char * GetMachineName(char *RemoteHost) {
   char *tmp,*tmp2;
   tmp = strstr(RemoteHost,".");
   if( tmp == NULL )
   {
     tmp = (char *)malloc(strlen(RemoteHost)+1);
     strcpy(tmp,RemoteHost);
     return tmp;
   }
   tmp2 = (char *)malloc(tmp-RemoteHost+1);
   strncpy(tmp2,RemoteHost,tmp-RemoteHost);
   return tmp2;
}

char *NameOfPort(char *Host) {
   struct hostent *hp;
   struct in_addr inp;
   
   inp.s_addr = inet_addr(Host);
   if( inp.s_addr == INADDR_NONE )
     return NULL;
   hp = gethostbyaddr((char *)&inp,4,AF_INET);
   if( hp == NULL)
     return NULL;
   return hp->h_name;
}

//------------------------------------------------------------
PServerInfo CreateServer(int port,int type) {
  PServerInfo SI;
  int len;

  SI = (PServerInfo) malloc(sizeof(ServerInfo));
  if( type == SOCK_STREAM )
    SI->sock = socket(AF_INET,type,getprotobyname("tcp")->p_proto);
  else if( type == SOCK_DGRAM )
    SI->sock = socket(AF_INET,type,getprotobyname("udp")->p_proto);
  else
    return NULL;
  if( SI->sock == -1 ) {
    free(SI);
    return NULL;
  }
  memset(&(SI->SAddr),0,sizeof(struct sockaddr_in));
  len = sizeof(struct sockaddr_in);
  if( getsockname(SI->sock,(struct sockaddr *)&(SI->SAddr),&len) == -1 ) {
    close(SI->sock);
    free(SI);
    return NULL;
  }
  
  len = 1;
  setsockopt(SI->sock,SOL_SOCKET,SO_REUSEADDR,(char *)&len,sizeof(len));

  SI->SAddr.sin_family = AF_INET;
  SI->SAddr.sin_port = htons(port);
  SI->SAddr.sin_addr.s_addr = 0;
  if( bind(SI->sock,(struct sockaddr *)&(SI->SAddr), sizeof(SI->SAddr)) == -1 ) {
    close(SI->sock);
    free(SI);
    return NULL;
  }
  return SI;
}

int ServerListen(PServerInfo SI) {
  if( SI == NULL )
    return -1;
  if( listen(SI->sock,5) == -1 )
    return -1;
  return 0;
}

PClientSocket ServerAcceptConnection(PServerInfo SI) {
  struct sockaddr sad;
  int len;
  int tmpsock;
  PClientSocket CS;

  if( SI == NULL)
    return NULL;
  len = sizeof(sad);
  tmpsock = accept(SI->sock,&sad,&len);
  if( tmpsock == -1 )
    return NULL;
  CS = (PClientSocket) malloc(sizeof(ClientSocket));
  CS->sock = tmpsock;
  memcpy(&CS->SAddr,&sad,sizeof(CS->SAddr));
  return CS;
}

void ServerDisconnect(PServerInfo SI) {
  if( SI == NULL )
    return;
  close(SI->sock);
}

//------------------------------------------------------------
PClientSocket ClientConnect(char *adrs,char *port) {
  struct servent *SE;
  struct sockaddr_in sin;
  struct hostent *HE;
  PClientSocket CS;
  
  CS = (PClientSocket) malloc(sizeof(ClientSocket));
  CS->sock = socket(AF_INET,SOCK_STREAM,getprotobyname("tcp")->p_proto);
  if( CS->sock == -1 ) {
    free(CS);
    return NULL;
  }
  sin.sin_family = AF_INET;
  SE = getservbyname(port,"tcp");
  if( SE == NULL)
    sin.sin_port = htons(atoi(port));
  else
    sin.sin_port = SE->s_port;
  sin.sin_addr.s_addr = inet_addr(adrs);
  if( sin.sin_addr.s_addr == INADDR_NONE ) {
    HE = gethostbyname(adrs);
    if( HE == NULL )
    {
      printf("Host inconnu\n");
      return NULL;
    }
    sin.sin_addr = *(struct in_addr *)(HE->h_addr_list[0]);
  }
  if( connect(CS->sock,(struct sockaddr *)(&sin),sizeof(sin)) == -1 ) {
    close(CS->sock);
    free(CS);
    return NULL;
  }
  return CS;
}

int ClientSend(PClientSocket CS,char *msg) {
  if( CS == NULL )
    return -1;
  return send(CS->sock,msg,strlen(msg),0);
}

//------------------------------------------------------------
int UDPSendBroadcast(PServerInfo SI,char *Text,int len,char *port) {
   struct sockaddr_in sin;
   int i,si;
   
   si = sizeof(int);
   if( getsockopt(SI->sock,SOL_SOCKET,SO_BROADCAST,&i,&si) == SOCKET_ERROR )
       return SOCKET_ERROR;
   if( i == 0 ) {
      i = 1;
     if(SetSocketOptions(SI->sock,SOL_SOCKET,SO_BROADCAST) == -1)
       return SOCKET_ERROR;
      if( getsockopt(SI->sock,SOL_SOCKET,SO_BROADCAST,&i,&si) == SOCKET_ERROR )
	return SOCKET_ERROR;
   }
   sin.sin_family = AF_INET;
   sin.sin_port = htons(GetPortByName(port,"udp"));
   sin.sin_addr.s_addr = INADDR_BROADCAST;

   i = sendto(SI->sock,Text,len,0,&sin,sizeof(sin));
   return i;
}

int UDPSendToAddr(PServerInfo SI,char *Text,int len,char *Addr,char *port) {
   int i,si;
   struct sockaddr_in sin;
   struct hostent *PHE;
   
   si = sizeof(int);
   sin.sin_addr.s_addr = inet_addr(Addr);
   if( sin.sin_addr.s_addr == INADDR_NONE ) {
      PHE = gethostbyname(Addr);
      if( PHE == NULL )
	return SOCKET_ERROR;
      sin.sin_addr = *(struct in_addr *)(PHE->h_addr_list[0]);
   }
   if( getsockopt(SI->sock,SOL_SOCKET,SO_BROADCAST,&i,&si) == SOCKET_ERROR )
      return SOCKET_ERROR;
   if( i == 0 ) {
      i = 1;
      if( getsockopt(SI->sock,SOL_SOCKET,SO_BROADCAST,&i,&si) == SOCKET_ERROR )
         return SOCKET_ERROR;
   }
   sin.sin_family = AF_INET;
   sin.sin_port = htons(GetPortByName(port,"udp"));
   i = sendto(SI->sock,Text,len,0,&sin,sizeof(sin));
   return i;
}

int UDPReceiveFrom(PServerInfo SI,char *Text,int len,char **ip,int Blocking) {
   struct sockaddr_in sin;
   int ssin;
   int i;
   struct hostent *hp;

   if(!Blocking)
   {
     fcntl(SI->sock,F_SETFL,O_NONBLOCK);
   }
   ssin = sizeof(sin);
   i = recvfrom(SI->sock,Text,len,0,&sin,&ssin);
   if( i == SOCKET_ERROR )
     return SOCKET_ERROR;

   hp = gethostbyaddr((char *)&sin.sin_addr,4,AF_INET);
   if( hp == NULL )
     return i;
   *ip = (char *)hp->h_name;

   return i;
}

int SetSocketOptions(int sock,int Level,int Opt)
{
  int value = 1;
  
  return setsockopt(sock,Level,Opt,(char *)&value,sizeof(int));
}


